function [] = PlotSimDataP(simData,whichExample)
% Plotting simulation data, version "P" (published).
% This function plots the signals that are also shown in the Balancing Cube
% example of the manuscript (plus additional signals).
%
% Inputs:
%   simData         simulation data
%   whichExample    Which simulation example?
%                       1: thermo-fluid
%                       2: Balancing Cube
%
% --
% Supplementary material (simulation examples) to:
%   Distributed Event-based State Estimation (technical report)
%   Event-based State Estimation: An Emulation-based Approach (journal)
%
% Copyright 2015-17 Max Planck Society. All rights reserved.
% 
% Sebastian Trimpe
% Max Planck Institute for Intelligent Systems
% Autonomous Motion Department
% strimpe(at)tuebingen.mpg.de
%
% Revision history
% [01.04.15, ST]    first version
% [04.08.15, ST]    full revision, new version to be published (first
%                   version ready)

close all;


%% General configuration

% Color order (not used for all plots)
%colOrd = {'b','r','c','m','y','k'};
colOrd = {'k','b','r','c','m','y','g'};


%% Configuration of plots
% Configuration of the individual plots

% 1) Plot of states and estimation errors
plot_statesAndErrors = true;    % true/false

% Indiced of what states to be plotted
p1_whichStates_BC = [2,5,13];   % Balancing Cube examples (1..14)
p1_whichStates_tf = [1,2,3,4];  % thermo-fluid examples (1..4)

% Whose agent's estimate is considered for computing errors
p1_whichAgent_BC = 2;           % Balancing Cube example (1..6)
p1_whichAgent_tf = 2;           % Balancing Cube example (1..2)


% 2) Plot of average communication rates (measurements and inputs)
% Plotted for all agents and all communication types.
plot_commRates = true;      % true/false

% horizon for moving-average filters
p2_MAhorz = 100;            % []=samples


% 3) Inter-agent error
% Plotted for all agents and all communication types.
plot_interAgentError = true;    % true/false

% Indiced of what states to be plotted
p3_whichStates_BC = [2,5,13];   % Balancing Cube examples (1..14)
p3_whichStates_tf = [1,2,3,4];  % thermo-fluid examples (1..4)

% Which agent used to compare against?
p3_whichAgent_BC = 5;           % Balancing Cube example (1..6)
p3_whichAgent_tf = 2;           % Balancing Cube example (1..2)


%% Some calculations
n_x = size(simData.x,2);
n_y = size(simData.y,2);
n_u = size(simData.u,2);
nA = length(simData.agent);

k_final = size(simData.x,1);

% Time signal
Ts = simData.params.p_sys.Ts;
tt = 0:Ts:((k_final-1)*Ts);


%% Plot 1: States and errors
% Plot of states x, estimation error epsilon, difference to centralized
% estimator e.
if plot_statesAndErrors
    
    if whichExample==1
        % thermo-fluid
        p1_whichStates = p1_whichStates_tf;
        p1_whichAgent = p1_whichAgent_tf;
    elseif whichExample==2
        % Balancing Cube
        p1_whichStates = p1_whichStates_BC;
        p1_whichAgent = p1_whichAgent_BC;
    else
        error('Wrong value for input argument whichExample.');
    end;
    
    fig1 = figure;
    set(fig1,'name','States and estimation errors');
    
    % plots
    nPlots = length(p1_whichStates);
    for iSt=1:nPlots
        % State x
        subplot(3,nPlots,iSt);
        plot(tt,simData.x(:,p1_whichStates(iSt)),colOrd{1});
        grid on;
        title(['Component ',num2str(p1_whichStates(iSt))]);
        ylabel('State x');
        
        % Estimation error epsilon
        subplot(3,nPlots,iSt+nPlots);
        plot(tt, simData.x(:,p1_whichStates(iSt)) ...
            -simData.agent(p1_whichAgent).xhat_full(:,p1_whichStates(iSt)), colOrd{1});
        grid on;
        ylabel(['Error \epsilon_',num2str(p1_whichAgent)]);
        
        % Difference to centralized estimator
        subplot(3,nPlots,iSt+2*nPlots);
        plot(tt, simData.estC.xhat_full(:,p1_whichStates(iSt)) ...
            -simData.agent(p1_whichAgent).xhat_full(:,p1_whichStates(iSt)), colOrd{1});
        grid on;
        ylabel(['Error e_',num2str(p1_whichAgent)]);
        xlabel('Time (s)');
    end;
    
end;


%% Plot 2: Communication rates
% Plot communication rates for all agents.
if plot_commRates
    
    fig2 = figure;
    set(fig2,'name','Communication rates');
    
    % plots
    for iA=1:nA     % for each agent
        subplot(1,nA,iA);
        
        % All measurements
        nS = size(simData.agent(iA).gamma_y,2);
        for iS=1:nS
            sig_filt = MAfilter(simData.agent(iA).gamma_y(:,iS), p2_MAhorz);
            plot(tt,sig_filt,colOrd{iS});
            grid on; hold on;
        end;
        % All inputs
        for iI=1:size(simData.agent(iA).gamma_u,2)
            sig_filt = MAfilter(simData.agent(iA).gamma_u(:,iI), p2_MAhorz);
            plot(tt,sig_filt,colOrd{iI+nS});
            grid on;
        end;
        
        xlabel('Time (s)');
        ylabel('Communication rates');
        title(['Agent ',num2str(iA)]);
    end;
    
end;



%% Plot 3: Inter-agent error
% Plot difference between two agents' estimates (inter-agent error)
if plot_interAgentError
    
    if whichExample==1
        % thermo-fluid
        p3_whichStates = p3_whichStates_tf;
        p3_whichAgent = p3_whichAgent_tf;
    elseif whichExample==2
        % Balancing Cube
        p3_whichStates = p3_whichStates_BC;
        p3_whichAgent = p3_whichAgent_BC;
    else
        error('Wrong value for input argument whichExample.');
    end;
    
    fig3 = figure;
    set(fig3,'name','Inter-agent error');
    
    % plots
    nCols = length(p3_whichStates);
    nRows = nA-1;
    for iSt=1:nPlots
        % Plot all inter-agent errors between agent i and p3_whichAgent
        
        iRow = -1;
        for iA=1:nA
            if iA==p3_whichAgent
                continue;
            end;
            iRow = iRow+1;
            
            subplot(nRows,nCols,iSt+iRow*nCols);
            
            plot(tt, simData.agent(iA).xhat_full(:,p3_whichStates(iSt)) ...
                -simData.agent(p3_whichAgent).xhat_full(:,p3_whichStates(iSt)), colOrd{1});
            grid on;
            if iA==1 || (iA==2 && p3_whichAgent==1)
                title(['Component ',num2str(p3_whichStates(iSt))]);
            end;
            ylabel(['Error e_{',num2str(iA),num2str(p3_whichAgent),'}']);
        end;
        
        xlabel('Time (s)');
    end;
    
    
end;

return;