function [res] = ConfigureSystemThermoFluid()
% Configure system model of thermo-fluid process (by Lunze et al).
% System has been used in:
%   [1] Lunze et al, Control Theory of Digitally Networked Dynamic Systems,
%   Cha. 5
%   [2] Sigurani et al, Experimental evaluation of two complementary
%   decentralized event-based control methods, Control Engineering
%   Practice, 2015.
%
% --
% Supplementary material (simulation examples) to:
%   Distributed Event-based State Estimation (technical report)
%   Event-based State Estimation: An Emulation-based Approach (journal)
%
% Copyright 2015-17 Max Planck Society. All rights reserved.
% 
% Sebastian Trimpe
% Max Planck Institute for Intelligent Systems
% Autonomous Motion Department
% strimpe(at)tuebingen.mpg.de
%
% Revision history
% [20.04.15, ST]   	first version
% [20.04.15, ST]    fix: Bc (factor 1e-3 missing)
%                   Treating sensor measurements individually
% [03.08.15, ST]    some clean up


%% Continuous-time model
% See above publications.
% State: [level tank 1, temperature 1, level 2, temperature 2]
Ac = 1e-3* [ -5.74   0       2.42    0
            -34.5   -8.58   43.9    5.44
            2.85    0       -5.01   0
            -46.5   5.58    39.2    -5.58];
Bc = 1e-3*diag([2.3 -38.9 2.59 35]);
Cc = eye(4);     % full state measurements

% Disturbance input
E1 = 1e-3*[0; 169];
E2 = 1e-3*[1.16; -20.7];
Ec = [E1, zeros(2,1); zeros(2,1), E2];

sys_c = ss(Ac,[Bc,Ec],Cc,[]);   % two types of inputs: control and disturbance input


%% Discrete-time model
Ts = 0.2;   % []=s
sys_d = c2d(sys_c,Ts);
[A,B_,C,~,~] = ssdata(sys_d);
B = B_(:,1:4);
E = B_(:,5:6);

% Initial condition
x_init = zeros(length(A),1);
%x_init = [0.001 0.05 0.001 0.05]'/10;


    
%% Agents and indices
% Consider 2 agents, corresponding to the 2 tanks.

% Number of agents:
numAgents = 2; 

% Measurement indices corresponding to 6 agents.
% Each entry (cell) corresponds to one agent.  For each agent, a cell 
% specifies what measurements belong to that agent.  Indeces are into rows
% of C.
%
% Example:
%   y_dims{2} = {4, 5};     % Agent 2 has 2 sensors: y4 and y5
%   y_dims{4} = {(6:7)'};   % Agent 4 has 1 sensor: y_{6:7}
%
% Indices (e.g. (6:7)') must be column vectors.
y_dims{1} = {1, 2};   % Agent 1: 2 sensor: y1, y2
y_dims{2} = {3, 4};     

% Inputs: same convention as for measurements
u_dims{1} = {(1:2)'};   % Agent 1: 1 actuator: u_{1,2}
u_dims{2} = {(3:4)'}; 


%% Noise
% All noise assumed uniform.
% levelNoiseLim = 0.002;   % noise on tank level, []=m
% tempNoiseLim = 0.05;     % noise on tank temparature, []=K
%
% Read off approximately from plots (experimental data) in Sigurani et al
levelNoiseLim = 0.002;   % noise on tank level, []=m; roughly 0.02/6/2
tempNoiseLim = 0.08;     % noise on tank temparature, []=K; roughly 2/12/2

% Measurement noise
w_lim = [levelNoiseLim, tempNoiseLim, levelNoiseLim, tempNoiseLim]';

% Input process noise
% None.
inpNoiseLim = 0.0;     % noise on input. []=1 
vInput_lim = inpNoiseLim*ones(4,1);


%% General disturbance 
dist.enabled = true;
dist.E = E;

% Disturbance signal
sig1 = zeros(12000,1);
sig2 = zeros(12000,1);
sig1(1000:3000) = 0.1;  % according to Sigurani et al.
sig2(5000:6000) = 0.25; % according to Sigurani et al.
dist.d_signal = [sig1, sig2];


%% Output results
res.A = A;
res.B = B;
res.C = C;
res.Ts = Ts;
res.x_init = x_init;
res.numAgents = numAgents;
res.n_x = length(A);
res.n_u = size(B,2);
res.n_y = size(C,1);
res.u_dims = u_dims;
res.y_dims = y_dims;
res.w_lim = w_lim;
res.vInput_lim = vInput_lim;
res.dist = dist;

% Estimator model: this is the model that the state estimator uses.
% This may be the model A,B,C above or a different one ("reduced order
% model")
res.est.flagReducedModel = false;   % no reduced model
res.est.A = A;
res.est.B = B;
res.est.Bminus = [];
res.est.C = C;
res.est.n_x = length(A);   % state dimension (reduced)
res.est.n_u = res.n_u;
res.est.x_init = x_init;
